# D3SLT - A Long-term Discriminative Single Shot Segmentation Tracker 



## Summary of the D3SLT tracker
The proposed tracker is based on the [D3Sv2](https://arxiv.org/abs/2112.11846) short-term tracker, extending it with long-term tracking capabilities. When the target is lost, the existing deep DCF module, combined with a motion model uncertainty is used for global re-detection, similarly as in the [FCLT](https://www.researchgate.net/publication/321326114_FCLT_-_A_Fully-Correlational_Long-Term_Tracker) tracker. Tracking confidence of the patch, which contains the segmentation output, is determined by the online [RT-MDNet](https://arxiv.org/abs/1808.08834) verifier. The tracker incorporates additional strategies for robust conflict resolution: (i) First, we perform periodic back-tracking of the re-detected target – the object is tracked backwards in time for a fixed number of frames to verify the validity of the re-detection. (ii) Second, a global search for targets, which are more likely to be the true target, is performed periodically during short-term tracking to revert from false positive tracks.

## Installation


#### Install dependencies

All the dependencies are listed in ``env.yml`` and can be installed using anaconda.

You can also run the installation script to install all the dependencies. You need to provide the conda install path (e.g. ~/anaconda3).
```bash
bash install.sh conda_install_path
```

For the RT-MDNet verifier module, the pre-trained network is available [here](https://github.com/IlchaeJung/RT-MDNet). The path in the ``external/RTMDnet/options.py`` should be set accordingly.


The pre-trained networks for D3Sv2 are accesible [here](https://drive.google.com/drive/folders/1QFo15WSlhRCvvzuqfqBfC9OS-sUG2iXk).

In ``ltr/models/bbox_reg/bbox_reg.py`` change the ``segm_net_path`` variable accordingly.

In ``pytracking/parameter/segm/default_params.py`` change the params.bbox_reg_net_path and params.resnet_octconv_path variables accordingly.



The tracker was tested on the Ubuntu 20.04 machine with a NVidia RTX 3060Ti graphics card and cudatoolkit version 11.2.

#### Test the tracker on VOT-18 to VOT-22 LT sequences

1.) Set the dataset variable in ``pytracking/run_d3s_vot20.py`` accordingly.

2.) Run the tracker on a sequence:
```bash
cd pytracking
python run_d3s_vot20.py --sequence <seq_name>
```

### Evaluate the tracker using VOT

We provide two wrappers for VOT evaluation: ``vot_wrapper_bbox.py`` and ``vot_wrapper_segm.py`` respectively. The first one outputs bounding-box results, while the second one ouputs segmentation masks.
