import os
import sys
import argparse

import cv2
import numpy as np

env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

from vot.dataset import VOTDataset
from vot.region.io import read_file
from vot.region.raster import calculate_overlaps
from vot.region import RegionType

from pytracking.run_d3s_vot20 import overlay_mask, overlay_bbox, overlay_circle, draw_progress_bar, draw_text


def show_results(sequence=None):

    tracker_name = 'd3sv2'
    vot_dataset_path = '/home/alan/workspace/vot/evals/vot20/sequences'
    vot_results_base_path = '/home/alan/workspace/vot/evals/vot20/results'

    dataset = VOTDataset(vot_dataset_path)
    results_base_dir = os.path.join(vot_results_base_path, tracker_name, 'baseline')

    if sequence is None:
        print('Error: sequence is not given')
        exit(-1)

    seq = dataset[sequence]

    gt = seq.groundtruth()

    results_path = os.path.join(results_base_dir, seq.name, '%s_%08d.txt' % (seq.name, 0))  # change zero to show different run
    results = read_file(results_path)

    overlaps = calculate_overlaps(gt, results, (seq.metadata('width'), seq.metadata('height')))

    img = cv2.imread(seq.frame(0).filename())

    m = seq.groundtruth(0).rasterize((0, 0, img.shape[1] - 1, img.shape[0] - 1)).astype(np.uint8)

    overlay_mask(img, m, a=0.4, contour_line_width=1, mask_color=(255, 0, 0))

    cv2.namedWindow('win', cv2.WINDOW_AUTOSIZE)
    cv2.imshow('win', img)
    cv2.waitKey(0)

    default_wait_interval = 10
    wait_interval = default_wait_interval

    i = 1
    while True:

        img = cv2.imread(seq.frame(i).filename())

        if results[i].type is RegionType.MASK:
            pred_mask = results[i].rasterize((0, 0, img.shape[1] - 1, img.shape[0] - 1)).astype(np.uint8)

            overlay_mask(img, pred_mask, a=0.2, contour_line_width=1)

        if gt[i].type is RegionType.MASK:
            gt_mask = gt[i].rasterize((0, 0, img.shape[1] - 1, img.shape[0] - 1)).astype(np.uint8)

            overlay_mask(img, gt_mask, a=-1, contour_line_width=1, mask_color=(255, 0, 0))

        draw_progress_bar(img, i, len(seq))
        draw_text(img, '%d/%d' % (i + 1, len(seq)), (10, 25))
        draw_text(img, '%.2f' % overlaps[i], (10, 45))

        cv2.imshow('win', img)
        key_ = cv2.waitKey(wait_interval)

        if wait_interval > 0:
            i = min(len(seq) - 1, i + 1)

        if key_ == 27:
            exit(0)
        elif key_ == 32:
            if wait_interval > 0:
                wait_interval = 0
            else:
                wait_interval = default_wait_interval
        elif key_ == 97:
            i = max(1, i - 1)
        elif key_ == 100:
            i = min(len(seq) - 1, i + 1)


def main():
    parser = argparse.ArgumentParser(description='Run tracker on sequence or dataset.')
    parser.add_argument('--sequence', type=str, default=None, help='Sequence number or name.')

    args = parser.parse_args()

    show_results(args.sequence)


if __name__ == '__main__':
    main()
