import vot_new as vot
import sys
import cv2
import numpy as np
import os
os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "0"

from pytracking.tracker.segm import Segm
from pytracking.parameter.segm import default_params as vot_params


def rect_to_poly(rect):
    x0 = rect[0]
    y0 = rect[1]
    x1 = rect[0] + rect[2]
    y1 = rect[1]
    x2 = rect[0] + rect[2]
    y2 = rect[1] + rect[3]
    x3 = rect[0]
    y3 = rect[1] + rect[3]
    return [x0, y0, x1, y1, x2, y2, x3, y3]

def mask_from_region(region, image_sz):
    if len(region) == 4:
        region = rect_to_poly(region)
    mask = np.zeros((image_sz[1], image_sz[0]), dtype=np.uint8)
    x_ = [region[i] for i in range(0, len(region), 2)]
    y_ = [region[i] for i in range(1, len(region), 2)]
    x0 = max(0, round(min(x_)))
    y0 = max(0, round(min(y_)))
    x1 = min(image_sz[0] - 1, round(max(x_)))
    y1 = min(image_sz[1] - 1, round(max(y_)))
    mask[y0:y1+1, x0:x1+1] = 1
    return mask

def make_full_size(x, output_sz):
    '''
    zero-pad input x (right and down) to match output_sz
    x: numpy array e.g., binary mask
    output_sz: size of the output [width, height]
    '''
    if x.shape[0] == output_sz[1] and x.shape[1] == output_sz[0]:
        return x
    pad_x = output_sz[0] - x.shape[1]
    if pad_x < 0:
        x = x[:, :x.shape[1] + pad_x]
        # padding has to be set to zero, otherwise pad function fails
        pad_x = 0
    pad_y = output_sz[1] - x.shape[0]
    if pad_y < 0:
        x = x[:x.shape[0] + pad_y, :]
        # padding has to be set to zero, otherwise pad function fails
        pad_y = 0
    return np.pad(x, ((0, pad_y), (0, pad_x)), 'constant', constant_values=0)

# MAIN
handle = vot.VOT("mask")
selection = handle.region()
imagefile = handle.frame()
if not imagefile:
    sys.exit(0)

params = vot_params.parameters()

image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB)

mask = make_full_size(selection, (image.shape[1], image.shape[0]))

params.save_mask = False

tracker = Segm(params)
tracker.initialize(image, [], init_mask=mask)

while True:
    imagefile = handle.frame()
    if not imagefile:
        break

    image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB)

    prediction = tracker.track(image)

    m = tracker.pred_mask
    if m is None:
        m = mask_from_region(prediction, (image.shape[1], image.shape[0]))

    handle.report(m, confidence=tracker.mdnet_score)
